local L = AceLibrary("AceLocale-2.2"):new("fmBuffCheck")

fmBuffCheck = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0", "AceComm-2.0", "FuBarPlugin-2.0")
local fmBuffCheck, self = fmBuffCheck, fmBuffCheck

fmBuffCheck.commPrefix = "fmbc"
fmBuffCheck.commDist = "GROUP"
fmBuffCheck.player = UnitName("player")

-- FuBar configuration
fmBuffCheck.hasIcon = true
fmBuffCheck.hideWithoutStandby = true
fmBuffCheck.defaultPosition = "MINIMAP"
fmBuffCheck.defaultMinimapPosition = 200
fmBuffCheck.hasNoText = true
fmBuffCheck.hasNoColor = true
fmBuffCheck.cannotDetachTooltip = true
fmBuffCheck.independentProfile = false
fmBuffCheck.blizzardTooltip = true

fmBuffCheck:RegisterDB("fmBuffCheckDB", "fmBuffCheckDBPC")
fmBuffCheck:RegisterDefaults("profile", { debugging = false, nagging = false} )

fmBuffCheck.options = {
	type = 'group',
	handler = fmBuffCheck,
	args = {
		debugMode = {
			order = 10,
			name = L["Debugging Mode"],
			desc = L["If enabled, additional messages are printed out to help debugging."],
			type = 'toggle',
			get = function() return self.db.profile.debugging end,
			set = function(v) self.db.profile.debugging = v end,
		},
		nagMode = {
			order = 20,
			name = L["Nagging Mode"],
			desc = L["If enabled, every person NOT having all buffs is being reminded by whisper."],
			type = 'toggle',
			get = function() return self.db.profile.nagging end,
			set = function(v) self.db.profile.nagging = v end,
		}
	}
}

fmBuffCheck.version = setmetatable({}, {__index = function(self, id)
	local t = {}
	self[id] = t
	return t
end})
	
fmBuffCheck.version = GetAddOnMetadata("fmBuffCheck", "field") 
	
fmBuffCheck.OnMenuRequest = fmBuffCheck.options

function fmBuffCheck:OnInitialize()
	self:SetCommPrefix(self.commPrefix)
	self:RegisterComm(self.commPrefix, self.commDist, "OnCommReceive")
	self:UpdateTooltip()
end

function fmBuffCheck:OnEnable()
	self:RegisterChatCommand('/fmBuffCheck', '/fmbc', {
		type = 'execute',
		func = function()
			self:CmdI2buf()
		end
	})
end

function fmBuffCheck:Send(cmd, res)
	self:SendCommMessage(self.commDist, cmd, res)
	self:OnCommReceive(self.commPrefix, self.player, self.commDist, cmd, res)
end

function fmBuffCheck:OnCommReceive(prefix, sender, distribution, cmd, res)
	
	if     cmd == "Q2VER" then self:CmdQ2ver(sender, cmd, res) -- received query for version info
	elseif cmd == "Q2BUF" then self:CmdQ2buf(sender, cmd, res) -- received query for buff info
	elseif cmd == "R2VER" then self:CmdR2ver(sender, cmd, res) -- received report of version info
	elseif cmd == "R2BUF" then self:CmdR2buf(sender, cmd, res) -- received report of buff info
	else                       self:CmdUnkwn(sender, cmd, res) -- received unknown command
	end
	
end

function fmBuffCheck:CmdI2ver()
	-- request an update of all versions from group members
	self:DebugPrint(L["Requesting version status of all party members..."])
	self:Send("Q2VER", false)
end

function fmBuffCheck:CmdI2buf()
	-- request an update of all buffs from group members
	self:DebugPrint(L["Requesting buff status of all party members..."])
	self:Send("Q2BUF", false)
end

function fmBuffCheck:CmdQ2ver(sender, cmd, res)
	 -- received query for version info
	self:DebugPrint((L["Version list request received from %s."]):format(sender))
	self:Send("R2VER", self.version)
end

function fmBuffCheck:CmdQ2buf(sender, cmd, res)
	-- received query for buff info
	self:DebugPrint((L["Buff list request received from %s."]):format(sender))
	self:GetAndSendBuffs()
end

function fmBuffCheck:CmdR2ver(sender, cmd, res)
	-- received report of version info
	self:DebugPrint((L["Version list result received from %s."]):format(sender))
	self:Print(sender .. ": V" .. res)
	--self:Send("R2VER", buff["o"], buff["d"], buff["w"], buff["f"])
end

function fmBuffCheck:CmdR2buf(sender, cmd, res)
	-- received report of buff info
	self:DebugPrint((L["Buff list result received from %s."]):format(sender))
	
	local b = {}
	local g = {}
	local f = {}
	local w = {}
	
	cnt = 1
	while (res[cnt .. "-type"]) do
		if (res[cnt .. "-type"] == "gbuff") then
			local btyp = self:BuffParse(res[cnt .. "-ttip"])
			if     (btyp == "b") then
				b = {["name"] = res[cnt .. "-name"], ["time"] = res[cnt .. "-time"]}
			elseif (btyp == "g") then
				g = {["name"] = res[cnt .. "-name"], ["time"] = res[cnt .. "-time"]}
			elseif (btyp == "f") then
				f = {["name"] = res[cnt .. "-name"], ["time"] = res[cnt .. "-time"]}
			end
		elseif (res[cnt .. "-type"] == "wbuff") then
			w = {["name"] = res[cnt .. "-name"], ["time"] = res[cnt .. "-time"]}
		end
		--self:Print(("%s: %s (%s)"):format(sender, res[cnt .. "-name"], fmBuffCheck:FormatTime(res[cnt .. "-time"])))
		--self:Print(res[cnt .. "-ttip"])
		--res[cnt .. "-type"] = "gbuff"
		--res[cnt .. "-name"] = GetPlayerBuffName(id)
		--res[cnt .. "-time"] = GetPlayerBuffTimeLeft(id)
		--res[cnt .. "-appl"] = GetPlayerBuffApplications(id)
		--res[cnt .. "-ttip"] = self:GetBuffTooltip(id)
		cnt = cnt + 1
	end
	
	self:Print(("%s: b:%s | g:%s | f:%s | w:%s"):format(sender, self:PP(b), self:PP(g), self:PP(f), self:PP(w)))
end

function fmBuffCheck:CmdUnkwn(sender, cmd, res)
	-- received unknown command
	self:DebugPrint((L["Unknown QUERY request received from %s: %s"]):format(sender, cmd))
end

function fmBuffCheck:GetAndSendBuffs()

	local cnt = 1
	local buffs = {}
	
	for num = 1, 40 do
		local id = GetPlayerBuff(num, "HELPFUL")
		if id > 0 then
			buffs[cnt .. "-type"] = "gbuff"
			buffs[cnt .. "-name"] = GetPlayerBuffName(id)
			buffs[cnt .. "-time"] = GetPlayerBuffTimeLeft(id)
			buffs[cnt .. "-appl"] = GetPlayerBuffApplications(id)
			buffs[cnt .. "-ttip"] = self:GetBuffTooltip(id)
			cnt = cnt + 1
		end
	end
	
	local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo(); 
	if hasMainHandEnchant then
		buffs[cnt .. "-type"] = "wbuff"
		buffs[cnt .. "-name"] = self:WeaponParse("MainHandSlot")
		buffs[cnt .. "-time"] = mainHandExpiration / 1000
		buffs[cnt .. "-appl"] = mainHandCharges
		buffs[cnt .. "-ttip"] = self:GetItemTooltip("MainHandSlot")
		cnt = cnt +1
	end
	if hasOffHandEnchant then
		buffs[cnt .. "-type"] = "wbuff"
		buffs[cnt .. "-name"] = self:WeaponParse("OffHandSlot")
		buffs[cnt .. "-time"] = offHandExpiration / 1000
		buffs[cnt .. "-appl"] = offHandCharges
		buffs[cnt .. "-ttip"] = self:GetItemTooltip("OffHandSlot")
		cnt = cnt +1
	end
	
	self:Send("R2BUF", buffs)
end

function fmBuffCheck:GetBuffTooltip(id)
	fmBuffCheckTooltip:ClearLines()
	fmBuffCheckTooltip:SetUnitBuff(self.player, id)
	local ret = ""
	
	for i = 1, fmBuffCheckTooltip:NumLines() do
		local txt = getglobal("fmBuffCheckTooltipTextLeft" .. i):GetText()
		ret = ret .. txt .. "\n"
    end
    return ret
end

function fmBuffCheck:GetItemTooltip(slot)
	fmBuffCheckTooltip:ClearLines()
	local slotId, _ = GetInventorySlotInfo(slot)
	fmBuffCheckTooltip:SetInventoryItem(self.player, slotId)
	local ret = ""
	
	for i = 1, fmBuffCheckTooltip:NumLines() do
		local txt = getglobal("fmBuffCheckTooltipTextLeft" .. i):GetText()
		ret = ret .. txt .. "\n"
    end
    return ret
end

function fmBuffCheck:BuffParse(ttip)
	if string.find(ttip, L["Battle Elixir"]  , 1, true) then return "b" end
	if string.find(ttip, L["Guardian Elixir"], 1, true) then return "g" end
	if string.find(ttip, L["Well Fed"]       , 1, true) then return "f"	end
    return "x"
end

function fmBuffCheck:WeaponParse(slot)
	fmBuffCheckTooltip:ClearLines()
	local slotId, _ = GetInventorySlotInfo(slot)
	fmBuffCheckTooltip:SetInventoryItem(self.player, slotId)
	return getglobal("fmBuffCheckTooltipTextLeft1"):GetText()
end

function fmBuffCheck:FormatTime(time)
	if time > 60 then
		return math.floor(time / 60) .. ":" .. ("%02d"):format(math.floor(time) % 60) .. "m"
	end
	return math.floor(time) .. "s"
end

function fmBuffCheck:PP(obj)
	local name = obj["name"]
	local timeleft = obj["time"]

	if (name == nil or name == "") then
		return string.format("|cffff0000%s|r", L["none"]) 
	end
	
	return string.format("|cff00ff00%s (%s)|r", name, self:FormatTime(timeleft)) 
end

function fmBuffCheck:Alert(s, b, g, f, w)
	if self.db.profile.nagging then
	
		local b_, g_, f_, w_, s_
	
		b_ = (b["name"] == nil or b["name"] == "")
		g_ = (g["name"] == nil or g["name"] == "")
		f_ = (f["name"] == nil or f["name"] == "")
		w_ = (w["name"] == nil or w["name"] == "")
		s_ = ""
	
		if (b_ or g_ or f_ or w_) then
			if b_ then
				s_ = s_ .. L["[offensive elixir]"]
			end
			if g_ then
				s_ = s_ .. L["[defensive elixir]"]
			end
			if f_ then
				s_ = s_ .. L["[food buff]"]
			end
			if w_ then
				s_ = s_ .. L["[weapon buff]"]
			end
			SendChatMessage(L["You are missing the following buffs: "] .. s_, "WHISPER", nil, s)
		end
	end
end

function fmBuffCheck:DebugPrint(msg)
	if self.db.profile.debugging then
		self.Print(msg)
	end
end

function fmBuffCheck:OnTooltipUpdate()
	GameTooltip:AddLine("fmBuffCheck")
	GameTooltip:AddLine(" ")
	GameTooltip:AddDoubleLine("Nagging:", (self.db.profile.nagging and YES or NO), 1, 1, 0, 1, 1, 1)
	GameTooltip:AddDoubleLine("Debugging:", (self.db.profile.debugging and YES or NO), 1, 1, 0, 1, 1, 1)
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine("|cffffff00" .. L["Click|r to initiate buff check."], 0.2, 1, 0.2)
	GameTooltip:AddLine("|cffffff00" .. L["Shift-Click|r to initiate version check."], 0.2, 1, 0.2)
end

function fmBuffCheck:OnClick()
	local shift, alt, ctrl = IsShiftKeyDown(), IsAltKeyDown(), IsControlKeyDown()
	if not shift and not alt and not ctrl then
		self:CmdI2buf()
	end
	if shift and not alt and not ctrl then
		self:CmdI2ver()
	end
end

function fmBuffCheck:IsGrouped()
	local inRaid = (GetNumRaidMembers() > 0)
	local inGroup = (GetNumPartyMembers() > 0)
	return (inRaid or inGroup)
end